# Generated by Django 2.1.7 on 2019-03-21 06:12

from django.db import migrations, models
from django.conf import settings


if settings.WITH_GIS:
    import django.contrib.gis.db.models.fields

    class Migration(migrations.Migration):

        initial = True

        dependencies = [
        ]

        operations = [
            migrations.CreateModel(
                name='GeometryModel',
                fields=[
                    ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                    ('name', models.CharField(max_length=255)),
                    ('geometry', django.contrib.gis.db.models.fields.GeometryField(srid=4326)),
                ],
                options={
                    'abstract': False,
                },
            ),
            migrations.CreateModel(
                name='PointModel',
                fields=[
                    ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                    ('name', models.CharField(max_length=255)),
                    ('geometry', django.contrib.gis.db.models.fields.PointField(srid=4326)),
                ],
                options={
                    'abstract': False,
                },
            ),
        ]
else:
    Migration = None
